@@ -13,14 +13,12 @@ You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
-// Copyright (c) 1996-2020 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2024 Live Networks, Inc.  All rights reserved.
 // 'Group sockets'
 // Implementation
 
 #include "Groupsock.hh"
 #include "GroupsockHelper.hh"
-//##### Eventually fix the following #include; we shouldn't know about tunnels
-#include "TunnelEncaps.hh"
 
 #ifndef NO_SSTREAM
 #include <sstream>
@@ -29,13 +27,13 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 
 ///////// OutputSocket //////////
 
-OutputSocket::OutputSocket(UsageEnvironment& env)
-  : Socket(env, 0 /* let kernel choose port */),
+OutputSocket::OutputSocket(UsageEnvironment& env, int family)
+  : Socket(env, 0 /* let kernel choose port */, family),
     fSourcePort(0), fLastSentTTL(256/*hack: a deliberately invalid value*/) {
 }
 
-OutputSocket::OutputSocket(UsageEnvironment& env, Port port)
-  : Socket(env, port),
+OutputSocket::OutputSocket(UsageEnvironment& env, Port port, int family)
+  : Socket(env, port, family),
     fSourcePort(0), fLastSentTTL(256/*hack: a deliberately invalid value*/) {
 }
 
@@ -55,7 +53,7 @@ Boolean OutputSocket::write(struct sockaddr_storage const& addressAndPort, u_int
   if (sourcePortNum() == 0) {
     // Now that we've sent a packet, we can find out what the
     // kernel chose as our ephemeral source port number:
-    if (!getSourcePort(env(), socketNum(), fSourcePort)) {
+    if (!getSourcePort(env(), socketNum(), addressAndPort.ss_family, fSourcePort)) {
       if (DebugLevel >= 1)
 	env() << *this
 	     << ": failed to get source port: "
@@ -78,7 +76,7 @@ Boolean OutputSocket
 ///////// destRecord //////////
 
 destRecord
-::destRecord(struct in_addr const& addr, Port const& port, u_int8_t ttl, unsigned sessionId,
+::destRecord(struct sockaddr_storage const& addr, Port const& port, u_int8_t ttl, unsigned sessionId,
 	     destRecord* next)
   : fNext(next), fGroupEId(addr, port.num(), ttl), fSessionId(sessionId) {
 }
@@ -92,18 +90,14 @@ destRecord::~destRecord() {
 
 NetInterfaceTrafficStats Groupsock::statsIncoming;
 NetInterfaceTrafficStats Groupsock::statsOutgoing;
-NetInterfaceTrafficStats Groupsock::statsRelayedIncoming;
-NetInterfaceTrafficStats Groupsock::statsRelayedOutgoing;
 
 // Constructor for a source-independent multicast group
-Groupsock::Groupsock(UsageEnvironment& env, struct in_addr const& groupAddr,
+Groupsock::Groupsock(UsageEnvironment& env, struct sockaddr_storage const& groupAddr,
 		     Port port, u_int8_t ttl)
-  : OutputSocket(env, port),
-    deleteIfNoMembers(False), isSlave(False),
+  : OutputSocket(env, port, groupAddr.ss_family),
     fDests(new destRecord(groupAddr, port, ttl, 0, NULL)),
     fIncomingGroupEId(groupAddr, port.num(), ttl) {
-
-  if (!socketJoinGroup(env, socketNum(), groupAddr.s_addr)) {
+  if (!socketJoinGroup(env, socketNum(), groupAddr)) {
     if (DebugLevel >= 1) {
       env << *this << ": failed to join group: "
 	  << env.getResultMsg() << "\n";
@@ -111,7 +105,7 @@ Groupsock::Groupsock(UsageEnvironment& env, struct in_addr const& groupAddr,
   }
 
   // Make sure we can get our source address:
-  if (ourIPAddress(env) == 0) {
+  if (!weHaveAnIPAddress(env)) {
     if (DebugLevel >= 0) { // this is a fatal error
       env << "Unable to determine our source address: "
 	  << env.getResultMsg() << "\n";
@@ -122,22 +116,20 @@ Groupsock::Groupsock(UsageEnvironment& env, struct in_addr const& groupAddr,
 }
 
 // Constructor for a source-specific multicast group
-Groupsock::Groupsock(UsageEnvironment& env, struct in_addr const& groupAddr,
-		     struct in_addr const& sourceFilterAddr,
+Groupsock::Groupsock(UsageEnvironment& env, struct sockaddr_storage const& groupAddr,
+		     struct sockaddr_storage const& sourceFilterAddr,
 		     Port port)
-  : OutputSocket(env, port),
-    deleteIfNoMembers(False), isSlave(False),
+  : OutputSocket(env, port, groupAddr.ss_family),
     fDests(new destRecord(groupAddr, port, 255, 0, NULL)),
     fIncomingGroupEId(groupAddr, sourceFilterAddr, port.num()) {
   // First try a SSM join.  If that fails, try a regular join:
-  if (!socketJoinGroupSSM(env, socketNum(), groupAddr.s_addr,
-			  sourceFilterAddr.s_addr)) {
+  if (!socketJoinGroupSSM(env, socketNum(), groupAddr, sourceFilterAddr)) {
     if (DebugLevel >= 3) {
       env << *this << ": SSM join failed: "
 	  << env.getResultMsg();
       env << " - trying regular join instead\n";
     }
-    if (!socketJoinGroup(env, socketNum(), groupAddr.s_addr)) {
+    if (!socketJoinGroup(env, socketNum(), groupAddr)) {
       if (DebugLevel >= 1) {
 	env << *this << ": failed to join group: "
 	     << env.getResultMsg() << "\n";
@@ -150,12 +142,11 @@ Groupsock::Groupsock(UsageEnvironment& env, struct in_addr const& groupAddr,
 
 Groupsock::~Groupsock() {
   if (isSSM()) {
-    if (!socketLeaveGroupSSM(env(), socketNum(), groupAddress().s_addr,
-			     sourceFilterAddress().s_addr)) {
-      socketLeaveGroup(env(), socketNum(), groupAddress().s_addr);
+    if (!socketLeaveGroupSSM(env(), socketNum(), groupAddress(), sourceFilterAddress())) {
+      socketLeaveGroup(env(), socketNum(), groupAddress());
     }
   } else {
-    socketLeaveGroup(env(), socketNum(), groupAddress().s_addr);
+    socketLeaveGroup(env(), socketNum(), groupAddress());
   }
 
   delete fDests;
@@ -164,14 +155,14 @@ Groupsock::~Groupsock() {
 }
 
 destRecord* Groupsock
-::createNewDestRecord(struct in_addr const& addr, Port const& port, u_int8_t ttl,
+::createNewDestRecord(struct sockaddr_storage const& addr, Port const& port, u_int8_t ttl,
 		      unsigned sessionId, destRecord* next) {
   // Default implementation:
   return new destRecord(addr, port, ttl, sessionId, next);
 }
 
 void
-Groupsock::changeDestinationParameters(struct in_addr const& newDestAddr,
+Groupsock::changeDestinationParameters(struct sockaddr_storage const& newDestAddr,
 				       Port newDestPort, int newDestTTL, unsigned sessionId) {
   destRecord* dest;
   for (dest = fDests; dest != NULL && dest->fSessionId != sessionId; dest = dest->fNext) {}
@@ -182,33 +173,36 @@ Groupsock::changeDestinationParameters(struct in_addr const& newDestAddr,
   }
 
   // "dest" is an existing 'destRecord' for this "sessionId"; change its values to the new ones:
-  struct in_addr destAddr = dest->fGroupEId.groupAddress();
-  if (newDestAddr.s_addr != 0) {
-    if (newDestAddr.s_addr != destAddr.s_addr
-	&& IsMulticastAddress(newDestAddr.s_addr)) {
+  struct sockaddr_storage destAddr = dest->fGroupEId.groupAddress();
+  if (!addressIsNull(newDestAddr)) {
+    // Replace "destAddr" with "newDestAddr"
+    if (!(newDestAddr == destAddr) && IsMulticastAddress(newDestAddr)) {
       // If the new destination is a multicast address, then we assume that
       // we want to join it also.  (If this is not in fact the case, then
       // call "multicastSendOnly()" afterwards.)
-      socketLeaveGroup(env(), socketNum(), destAddr.s_addr);
-      socketJoinGroup(env(), socketNum(), newDestAddr.s_addr);
+      socketLeaveGroup(env(), socketNum(), destAddr);
+      socketJoinGroup(env(), socketNum(), newDestAddr);
     }
-    destAddr.s_addr = newDestAddr.s_addr;
+    destAddr = newDestAddr;
   }
 
   portNumBits destPortNum = dest->fGroupEId.portNum();
   if (newDestPort.num() != 0) {
-    if (newDestPort.num() != destPortNum
-	&& IsMulticastAddress(destAddr.s_addr)) {
+    // Replace "destPort" with "newDestPort"
+    if (newDestPort.num() != destPortNum && IsMulticastAddress(destAddr)) {
       // Also bind to the new port number:
       changePort(newDestPort);
       // And rejoin the multicast group:
-      socketJoinGroup(env(), socketNum(), destAddr.s_addr);
+      socketJoinGroup(env(), socketNum(), destAddr);
     }
     destPortNum = newDestPort.num();
   }
 
   u_int8_t destTTL = ttl();
-  if (newDestTTL != ~0) destTTL = (u_int8_t)newDestTTL;
+  if (newDestTTL != ~0) {
+    // Replace "destTTL" with "newDestTTL"
+    destTTL = (u_int8_t)newDestTTL;
+  }
 
   dest->fGroupEId = GroupEId(destAddr, destPortNum, destTTL);
 
@@ -217,20 +211,21 @@ Groupsock::changeDestinationParameters(struct in_addr const& newDestAddr,
 }
 
 unsigned Groupsock
-::lookupSessionIdFromDestination(struct sockaddr_in const& destAddrAndPort) const {
+::lookupSessionIdFromDestination(struct sockaddr_storage const& destAddrAndPort) const {
   destRecord* dest = lookupDestRecordFromDestination(destAddrAndPort);
   if (dest == NULL) return 0;
 
   return dest->fSessionId;
 }
 
-void Groupsock::addDestination(struct in_addr const& addr, Port const& port, unsigned sessionId) {
+void Groupsock::addDestination(struct sockaddr_storage const& addr, Port const& port,
+			       unsigned sessionId) {
   // Default implementation:
   // If there's no existing 'destRecord' with the same "addr", "port", and "sessionId", add a new one:
   for (destRecord* dest = fDests; dest != NULL; dest = dest->fNext) {
-    if (sessionId == dest->fSessionId
-	&& addr.s_addr == dest->fGroupEId.groupAddress().s_addr
-	&& port.num() == dest->fGroupEId.portNum()) {
+    if (dest->fSessionId == sessionId &&
+	dest->fGroupEId.groupAddress() == addr &&
+	dest->fGroupEId.portNum() == port.num()) {
       return;
     }
   }
@@ -251,22 +246,19 @@ void Groupsock::multicastSendOnly() {
   // We disable this code for now, because - on some systems - leaving the multicast group seems to cause sent packets
   // to not be received by other applications (at least, on the same host).
 #if 0
-  socketLeaveGroup(env(), socketNum(), fIncomingGroupEId.groupAddress().s_addr);
+  socketLeaveGroup(env(), socketNum(), groupAddress(r);
   for (destRecord* dests = fDests; dests != NULL; dests = dests->fNext) {
-    socketLeaveGroup(env(), socketNum(), dests->fGroupEId.groupAddress().s_addr);
+    socketLeaveGroup(env(), socketNum(), dests->fGroupEId.groupAddress(r);
   }
 #endif
 }
 
-Boolean Groupsock::output(UsageEnvironment& env, unsigned char* buffer, unsigned bufferSize,
-			  DirectedNetInterface* interfaceNotToFwdBackTo) {
+Boolean Groupsock::output(UsageEnvironment& env, unsigned char* buffer, unsigned bufferSize) {
   do {
     // First, do the datagram send, to each destination:
     Boolean writeSuccess = True;
     for (destRecord* dests = fDests; dests != NULL; dests = dests->fNext) {
-      MAKE_SOCKADDR_IN(destAddressAndPort, dests->fGroupEId.groupAddress().s_addr, dests->fGroupEId.portNum()); // later update for IPv6
-      if (!write((struct sockaddr_storage const&)destAddressAndPort, dests->fGroupEId.ttl(),
-		 buffer, bufferSize)) {
+      if (!write(dests->fGroupEId.groupAddress(), dests->fGroupEId.ttl(), buffer, bufferSize)) {
 	writeSuccess = False;
 	break;
       }
@@ -275,22 +267,8 @@ Boolean Groupsock::output(UsageEnvironment& env, unsigned char* buffer, unsigned
     statsOutgoing.countPacket(bufferSize);
     statsGroupOutgoing.countPacket(bufferSize);
 
-    // Then, forward to our members:
-    int numMembers = 0;
-    if (!members().IsEmpty()) {
-      numMembers =
-	outputToAllMembersExcept(interfaceNotToFwdBackTo,
-				 ttl(), buffer, bufferSize,
-				 ourIPAddress(env));
-      if (numMembers < 0) break;
-    }
-
     if (DebugLevel >= 3) {
-      env << *this << ": wrote " << bufferSize << " bytes, ttl " << (unsigned)ttl();
-      if (numMembers > 0) {
-	env << "; relayed to " << numMembers << " members";
-      }
-      env << "\n";
+      env << *this << ": wrote " << bufferSize << " bytes, ttl " << (unsigned)ttl() << "\n";
     }
     return True;
   } while (0);
@@ -306,14 +284,10 @@ Boolean Groupsock::output(UsageEnvironment& env, unsigned char* buffer, unsigned
 Boolean Groupsock::handleRead(unsigned char* buffer, unsigned bufferMaxSize,
 			      unsigned& bytesRead,
 			      struct sockaddr_storage& fromAddressAndPort) {
-  // Read data from the socket, and relay it across any attached tunnels
-  //##### later make this code more general - independent of tunnels
+  bytesRead = 0; // initially
 
-  bytesRead = 0;
-
-  int maxBytesToRead = bufferMaxSize - TunnelEncapsulationTrailerMaxSize;
   int numBytes = readSocket(env(), socketNum(),
-			    buffer, maxBytesToRead, fromAddressAndPort);
+			    buffer, bufferMaxSize, fromAddressAndPort);
   if (numBytes < 0) {
     if (DebugLevel >= 0) { // this is a fatal error
       UsageEnvironment::MsgString msg = strDup(env().getResultMsg());
@@ -324,38 +298,17 @@ Boolean Groupsock::handleRead(unsigned char* buffer, unsigned bufferMaxSize,
   }
 
   // If we're a SSM group, make sure the source address matches:
-  if (isSSM()
-      && fromAddressAndPort.ss_family == AF_INET
-      && ((struct sockaddr_in*)&fromAddressAndPort)->sin_addr.s_addr != sourceFilterAddress().s_addr) {
-    return True;
-  }
+  if (isSSM() && !(fromAddressAndPort == sourceFilterAddress())) return True;
 
   // We'll handle this data.
-  // Also write it (with the encapsulation trailer) to each member,
-  // unless the packet was originally sent by us to begin with.
   bytesRead = numBytes;
 
-  int numMembers = 0;
   if (!wasLoopedBackFromUs(env(), fromAddressAndPort)) {
-    statsIncoming.countPacket(numBytes);
-    statsGroupIncoming.countPacket(numBytes);
-    numMembers =
-      outputToAllMembersExcept(NULL, ttl(),
-			       buffer, bytesRead,
-			       fromAddressAndPort.ss_family == AF_INET
-			       ? ((struct sockaddr_in*)&fromAddressAndPort)->sin_addr.s_addr
-			       : 0);
-    if (numMembers > 0) {
-      statsRelayedIncoming.countPacket(numBytes);
-      statsGroupRelayedIncoming.countPacket(numBytes);
-    }
+    statsIncoming.countPacket(bytesRead);
+    statsGroupIncoming.countPacket(bytesRead);
   }
   if (DebugLevel >= 3) {
-    env() << *this << ": read " << bytesRead << " bytes from " << AddressString(fromAddressAndPort).val() << ", port " << ntohs(portNum(fromAddressAndPort));
-    if (numMembers > 0) {
-      env() << "; relayed to " << numMembers << " members";
-    }
-    env() << "\n";
+    env() << *this << ": read " << bytesRead << " bytes from " << AddressString(fromAddressAndPort).val() << ", port " << ntohs(portNum(fromAddressAndPort)) << "\n";
   }
 
   return True;
@@ -366,7 +319,7 @@ Boolean Groupsock::wasLoopedBackFromUs(UsageEnvironment& env,
   if (fromAddressAndPort.ss_family != AF_INET) return False; // later update for IPv6
   
   struct sockaddr_in const& fromAddressAndPort4 = (struct sockaddr_in const&)fromAddressAndPort;
-  if (fromAddressAndPort4.sin_addr.s_addr == ourIPAddress(env) ||
+  if (fromAddressAndPort4.sin_addr.s_addr == ourIPv4Address(env) ||
       fromAddressAndPort4.sin_addr.s_addr == 0x7F000001/*127.0.0.1*/) {
     if (portNum(fromAddressAndPort) == sourcePortNum()) {
 #ifdef DEBUG_LOOPBACK_CHECKING
@@ -382,13 +335,14 @@ Boolean Groupsock::wasLoopedBackFromUs(UsageEnvironment& env,
 }
 
 destRecord* Groupsock
-::lookupDestRecordFromDestination(struct sockaddr_in const& destAddrAndPort) const {
+::lookupDestRecordFromDestination(struct sockaddr_storage const& targetAddrAndPort) const {
   for (destRecord* dest = fDests; dest != NULL; dest = dest->fNext) {
-    if (destAddrAndPort.sin_addr.s_addr == dest->fGroupEId.groupAddress().s_addr
-	&& destAddrAndPort.sin_port == dest->fGroupEId.portNum()) {
+    if (dest->fGroupEId.groupAddress() == targetAddrAndPort &&
+	dest->fGroupEId.portNum() == portNum(targetAddrAndPort)) {
       return dest;
     }
   }
+
   return NULL;
 }
 
@@ -407,88 +361,6 @@ void Groupsock::removeDestinationFrom(destRecord*& dests, unsigned sessionId) {
   }
 }
 
-int Groupsock::outputToAllMembersExcept(DirectedNetInterface* exceptInterface,
-					u_int8_t ttlToFwd,
-					unsigned char* data, unsigned size,
-					netAddressBits sourceAddr) {
-  // Don't forward TTL-0 packets
-  if (ttlToFwd == 0) return 0;
-
-  DirectedNetInterfaceSet::Iterator iter(members());
-  unsigned numMembers = 0;
-  DirectedNetInterface* interf;
-  while ((interf = iter.next()) != NULL) {
-    // Check whether we've asked to exclude this interface:
-    if (interf == exceptInterface)
-      continue;
-
-    // Check that the packet's source address makes it OK to
-    // be relayed across this interface:
-    UsageEnvironment& saveEnv = env();
-    // because the following call may delete "this"
-    if (!interf->SourceAddrOKForRelaying(saveEnv, sourceAddr)) {
-      if (strcmp(saveEnv.getResultMsg(), "") != 0) {
-				// Treat this as a fatal error
-	return -1;
-      } else {
-	continue;
-      }
-    }
-
-    if (numMembers == 0) {
-      // We know that we're going to forward to at least one
-      // member, so fill in the tunnel encapsulation trailer.
-      // (Note: Allow for it not being 4-byte-aligned.)
-      TunnelEncapsulationTrailer* trailerInPacket
-	= (TunnelEncapsulationTrailer*)&data[size];
-      TunnelEncapsulationTrailer* trailer;
-
-      Boolean misaligned = ((uintptr_t)trailerInPacket & 3) != 0;
-      unsigned trailerOffset;
-      u_int8_t tunnelCmd;
-      if (isSSM()) {
-	// add an 'auxilliary address' before the trailer
-	trailerOffset = TunnelEncapsulationTrailerAuxSize;
-	tunnelCmd = TunnelDataAuxCmd;
-      } else {
-	trailerOffset = 0;
-	tunnelCmd = TunnelDataCmd;
-      }
-      unsigned trailerSize = TunnelEncapsulationTrailerSize + trailerOffset;
-      unsigned tmpTr[TunnelEncapsulationTrailerMaxSize];
-      if (misaligned) {
-	trailer = (TunnelEncapsulationTrailer*)&tmpTr;
-      } else {
-	trailer = trailerInPacket;
-      }
-      trailer += trailerOffset;
-
-      if (fDests != NULL) {
-	trailer->address() = fDests->fGroupEId.groupAddress().s_addr;
-	Port destPort(ntohs(fDests->fGroupEId.portNum()));
-	trailer->port() = destPort; // structure copy
-      }
-      trailer->ttl() = ttlToFwd;
-      trailer->command() = tunnelCmd;
-
-      if (isSSM()) {
-	trailer->auxAddress() = sourceFilterAddress().s_addr;
-      }
-
-      if (misaligned) {
-	memmove(trailerInPacket, trailer-trailerOffset, trailerSize);
-      }
-
-      size += trailerSize;
-    }
-
-    interf->write(data, size);
-    ++numMembers;
-  }
-
-  return numMembers;
-}
-
 UsageEnvironment& operator<<(UsageEnvironment& s, const Groupsock& g) {
   UsageEnvironment& s1 = s << timestampString() << " Groupsock("
 			   << g.socketNum() << ": "
@@ -586,15 +458,15 @@ static Groupsock* getGroupsockBySocket(UsageEnvironment& env, int sock) {
 
 Groupsock*
 GroupsockLookupTable::Fetch(UsageEnvironment& env,
-			    netAddressBits groupAddress,
+			    struct sockaddr_storage const& groupAddress,
 			    Port port, u_int8_t ttl,
 			    Boolean& isNew) {
   isNew = False;
   Groupsock* groupsock;
   do {
-    groupsock = (Groupsock*) fTable.Lookup(groupAddress, (~0), port);
+    groupsock = (Groupsock*) fTable.Lookup(groupAddress, port);
     if (groupsock == NULL) { // we need to create one:
-      groupsock = AddNew(env, groupAddress, (~0), port, ttl);
+      groupsock = AddNew(env, groupAddress, nullAddress(), port, ttl);
       if (groupsock == NULL) break;
       isNew = True;
     }
@@ -605,8 +477,8 @@ GroupsockLookupTable::Fetch(UsageEnvironment& env,
 
 Groupsock*
 GroupsockLookupTable::Fetch(UsageEnvironment& env,
-			    netAddressBits groupAddress,
-			    netAddressBits sourceFilterAddr, Port port,
+			    struct sockaddr_storage const& groupAddress,
+			    struct sockaddr_storage const& sourceFilterAddr, Port port,
 			    Boolean& isNew) {
   isNew = False;
   Groupsock* groupsock;
@@ -624,13 +496,13 @@ GroupsockLookupTable::Fetch(UsageEnvironment& env,
 }
 
 Groupsock*
-GroupsockLookupTable::Lookup(netAddressBits groupAddress, Port port) {
-  return (Groupsock*) fTable.Lookup(groupAddress, (~0), port);
+GroupsockLookupTable::Lookup(struct sockaddr_storage const& groupAddress, Port port) {
+  return (Groupsock*) fTable.Lookup(groupAddress, port);
 }
 
 Groupsock*
-GroupsockLookupTable::Lookup(netAddressBits groupAddress,
-			     netAddressBits sourceFilterAddr, Port port) {
+GroupsockLookupTable::Lookup(struct sockaddr_storage const& groupAddress,
+			     struct sockaddr_storage const& sourceFilterAddr, Port port) {
   return (Groupsock*) fTable.Lookup(groupAddress, sourceFilterAddr, port);
 }
 
@@ -640,26 +512,22 @@ Groupsock* GroupsockLookupTable::Lookup(UsageEnvironment& env, int sock) {
 
 Boolean GroupsockLookupTable::Remove(Groupsock const* groupsock) {
   unsetGroupsockBySocket(groupsock);
-  return fTable.Remove(groupsock->groupAddress().s_addr,
-		       groupsock->sourceFilterAddress().s_addr,
-		       groupsock->port());
+
+  return fTable.Remove(groupsock->groupAddress(), groupsock->sourceFilterAddress(), groupsock->port());
 }
 
 Groupsock* GroupsockLookupTable::AddNew(UsageEnvironment& env,
-					netAddressBits groupAddress,
-					netAddressBits sourceFilterAddress,
+					struct sockaddr_storage const& groupAddress,
+					struct sockaddr_storage const& sourceFilterAddress,
 					Port port, u_int8_t ttl) {
   Groupsock* groupsock;
   do {
-    struct in_addr groupAddr; groupAddr.s_addr = groupAddress;
-    if (sourceFilterAddress == netAddressBits(~0)) {
+    if (addressIsNull(sourceFilterAddress)) {
       // regular, ISM groupsock
-      groupsock = new Groupsock(env, groupAddr, port, ttl);
+      groupsock = new Groupsock(env, groupAddress, port, ttl);
     } else {
       // SSM groupsock
-      struct in_addr sourceFilterAddr;
-      sourceFilterAddr.s_addr = sourceFilterAddress;
-      groupsock = new Groupsock(env, groupAddr, sourceFilterAddr, port);
+      groupsock = new Groupsock(env, groupAddress, sourceFilterAddress, port);
     }
 
     if (groupsock == NULL || groupsock->socketNum() < 0) break;
