@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2021 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2024 Live Networks, Inc.  All rights reserved.
 // A 'ServerMediaSubsession' object that creates new, unicast, "RTPSink"s
 // on demand.
 // Implementation
@@ -28,7 +28,8 @@ OnDemandServerMediaSubsession
 				portNumBits initialPortNum,
 				Boolean multiplexRTCPWithRTP)
   : ServerMediaSubsession(env),
-    fSDPLines(NULL), fReuseFirstSource(reuseFirstSource),
+    fSDPLines(NULL), fMIKEYStateMessage(NULL), fMIKEYStateMessageSize(0),
+    fReuseFirstSource(reuseFirstSource),
     fMultiplexRTCPWithRTP(multiplexRTCPWithRTP), fLastStreamToken(NULL),
     fAppHandlerTask(NULL), fAppHandlerClientData(NULL) {
   fDestinationsHashTable = HashTable::create(ONE_WORD_HASH_KEYS);
@@ -43,6 +44,7 @@ OnDemandServerMediaSubsession
 }
 
 OnDemandServerMediaSubsession::~OnDemandServerMediaSubsession() {
+  delete[] fMIKEYStateMessage;
   delete[] fSDPLines;
 
   // Clean out the destinations hash table:
@@ -69,10 +71,16 @@ OnDemandServerMediaSubsession::sdpLines(int addressFamily) {
     Groupsock* dummyGroupsock = createGroupsock(nullAddress(addressFamily), 0);
     unsigned char rtpPayloadType = 96 + trackNumber()-1; // if dynamic
     RTPSink* dummyRTPSink = createNewRTPSink(dummyGroupsock, rtpPayloadType, inputSource);
-    if (dummyRTPSink != NULL && dummyRTPSink->estimatedBitrate() > 0) estBitrate = dummyRTPSink->estimatedBitrate();
+    if (dummyRTPSink != NULL) {
+      if (fParentSession->streamingUsesSRTP) {
+	fMIKEYStateMessage = dummyRTPSink->setupForSRTP(fParentSession->streamingIsEncrypted,
+							fMIKEYStateMessageSize);
+      }
 
-    setSDPLinesFromRTPSink(dummyRTPSink, inputSource, estBitrate);
-    Medium::close(dummyRTPSink);
+      if (dummyRTPSink->estimatedBitrate() > 0) estBitrate = dummyRTPSink->estimatedBitrate();
+      setSDPLinesFromRTPSink(dummyRTPSink, inputSource, estBitrate);
+      Medium::close(dummyRTPSink);
+    }
     delete dummyGroupsock;
     closeStreamSource(inputSource);
   }
@@ -88,6 +96,7 @@ void OnDemandServerMediaSubsession
 		      int tcpSocketNum,
 		      unsigned char rtpChannelId,
 		      unsigned char rtcpChannelId,
+		      TLSState* tlsState,
 		      struct sockaddr_storage& destinationAddress,
 		      u_int8_t& /*destinationTTL*/,
 		      Boolean& isMulticast,
@@ -166,7 +175,12 @@ void OnDemandServerMediaSubsession
 	unsigned char rtpPayloadType = 96 + trackNumber()-1; // if dynamic
 	rtpSink = mediaSource == NULL ? NULL
 	  : createNewRTPSink(rtpGroupsock, rtpPayloadType, mediaSource);
-	if (rtpSink != NULL && rtpSink->estimatedBitrate() > 0) streamBitrate = rtpSink->estimatedBitrate();
+	if (rtpSink != NULL) {
+	  if (fParentSession->streamingUsesSRTP) {
+	    rtpSink->setupForSRTP(fMIKEYStateMessage, fMIKEYStateMessageSize);
+	  }
+	  if (rtpSink->estimatedBitrate() > 0) streamBitrate = rtpSink->estimatedBitrate();
+	}
       }
 
       // Turn off the destinations for each groupsock.  They'll get set later
@@ -195,7 +209,7 @@ void OnDemandServerMediaSubsession
   if (tcpSocketNum < 0) { // UDP
     destinations = new Destinations(destinationAddress, clientRTPPort, clientRTCPPort);
   } else { // TCP
-    destinations = new Destinations(tcpSocketNum, rtpChannelId, rtcpChannelId);
+    destinations = new Destinations(tcpSocketNum, rtpChannelId, rtcpChannelId, tlsState);
   }
   fDestinationsHashTable->Add((char const*)clientSessionId, destinations);
 }
@@ -316,7 +330,7 @@ FramedSource* OnDemandServerMediaSubsession::getStreamSource(void* streamToken)
 
 void OnDemandServerMediaSubsession
 ::getRTPSinkandRTCP(void* streamToken,
-		    RTPSink const*& rtpSink, RTCPInstance const*& rtcp) {
+		    RTPSink*& rtpSink, RTCPInstance*& rtcp) {
   if (streamToken == NULL) {
     rtpSink = NULL;
     rtcp = NULL;
@@ -419,8 +433,6 @@ void OnDemandServerMediaSubsession
 
 void OnDemandServerMediaSubsession
 ::setSDPLinesFromRTPSink(RTPSink* rtpSink, FramedSource* inputSource, unsigned estBitrate) {
-  if (rtpSink == NULL) return;
-
   char const* mediaType = rtpSink->sdpMediaType();
   unsigned char rtpPayloadType = rtpSink->rtpPayloadType();
   struct sockaddr_storage const& addressForSDP = rtpSink->groupsockBeingUsed().groupAddress();
@@ -428,25 +440,28 @@ void OnDemandServerMediaSubsession
 
   AddressString ipAddressStr(addressForSDP);
   char* rtpmapLine = rtpSink->rtpmapLine();
+  char* keyMgmtLine = rtpSink->keyMgmtLine();
   char const* rtcpmuxLine = fMultiplexRTCPWithRTP ? "a=rtcp-mux\r\n" : "";
   char const* rangeLine = rangeSDPLine();
   char const* auxSDPLine = getAuxSDPLine(rtpSink, inputSource);
   if (auxSDPLine == NULL) auxSDPLine = "";
 
   char const* const sdpFmt =
-    "m=%s %u RTP/AVP %d\r\n"
+    "m=%s %u RTP/%sAVP %d\r\n"
     "c=IN %s %s\r\n"
     "b=AS:%u\r\n"
     "%s"
     "%s"
     "%s"
     "%s"
+    "%s"
     "a=control:%s\r\n";
   unsigned sdpFmtSize = strlen(sdpFmt)
-    + strlen(mediaType) + 5 /* max short len */ + 3 /* max char len */
+    + strlen(mediaType) + 5 /* max short len */ + 1 + 3 /* max char len */
     + 3/*IP4 or IP6*/ + strlen(ipAddressStr.val())
     + 20 /* max int len */
     + strlen(rtpmapLine)
+    + strlen(keyMgmtLine)
     + strlen(rtcpmuxLine)
     + strlen(rangeLine)
     + strlen(auxSDPLine)
@@ -455,15 +470,17 @@ void OnDemandServerMediaSubsession
   sprintf(sdpLines, sdpFmt,
 	  mediaType, // m= <media>
 	  portNumForSDP, // m= <port>
+	  fParentSession->streamingUsesSRTP ? "S" : "",
 	  rtpPayloadType, // m= <fmt list>
 	  addressForSDP.ss_family == AF_INET ? "IP4" : "IP6", ipAddressStr.val(), // c= address
 	  estBitrate, // b=AS:<bandwidth>
 	  rtpmapLine, // a=rtpmap:... (if present)
+	  keyMgmtLine, // a=key-mgmt:... (if present)
 	  rtcpmuxLine, // a=rtcp-mux:... (if present)
 	  rangeLine, // a=range:... (if present)
 	  auxSDPLine, // optional extra SDP line
 	  trackId()); // a=control:<track-id>
-  delete[] (char*)rangeLine; delete[] rtpmapLine;
+  delete[] (char*)rangeLine; delete[] keyMgmtLine; delete[] rtpmapLine;
 
   delete[] fSDPLines; fSDPLines = strDup(sdpLines);
   delete[] sdpLines;
@@ -513,20 +530,20 @@ void StreamState
     // Create (and start) a 'RTCP instance' for this RTP sink:
     fRTCPInstance = fMaster.createRTCP(fRTCPgs, fTotalBW, (unsigned char*)fMaster.fCNAME, fRTPSink);
         // Note: This starts RTCP running automatically
-    fRTCPInstance->setAppHandler(fMaster.fAppHandlerTask, fMaster.fAppHandlerClientData);
+    if (fRTCPInstance != NULL) fRTCPInstance->setAppHandler(fMaster.fAppHandlerTask, fMaster.fAppHandlerClientData);
   }
 
   if (dests->isTCP) {
     // Change RTP and RTCP to use the TCP socket instead of UDP:
     if (fRTPSink != NULL) {
-      fRTPSink->addStreamSocket(dests->tcpSocketNum, dests->rtpChannelId);
+      fRTPSink->addStreamSocket(dests->tcpSocketNum, dests->rtpChannelId, dests->tlsState);
       RTPInterface
 	::setServerRequestAlternativeByteHandler(fRTPSink->envir(), dests->tcpSocketNum,
 						 serverRequestAlternativeByteHandler, serverRequestAlternativeByteHandlerClientData);
         // So that we continue to handle RTSP commands from the client
     }
     if (fRTCPInstance != NULL) {
-      fRTCPInstance->addStreamSocket(dests->tcpSocketNum, dests->rtcpChannelId);
+      fRTCPInstance->addStreamSocket(dests->tcpSocketNum, dests->rtcpChannelId, dests->tlsState);
 
       struct sockaddr_storage tcpSocketNumAsAddress; // hack
       tcpSocketNumAsAddress.ss_family = AF_INET;
